/**** A V R  A P P L I C A T I O N  NOTE 1 3 4 ************************** 
 *
 * Title:           Real Time Clock
 * Version:         1.1
 * Last Updated:    12.09.00
 * Target:          ATmega103 (All AVR Devices with secondary external oscillator)
 *
 * Support E-mail:  avr@atmel.com
 *
 * Description      
 * This application note shows how to implement a Real Time Clock utilizing a secondary 
 * external oscilator. Included a test program that performs this function, which keeps
 * track of time, date, month, and year with auto leap-year configuration. 8 LEDs are used
 * to display the RTC. The 1st LED flashes every second, the next six represents the
 * minute, and the 8th LED represents the hour.
 *
 * Change log
 * 	12.10.98		v1.01	JB 	Initial version
 *	12.09.00		v1.10	JB	Optimized code, commented out 100 year leap check
 *		 						Changed access of global variables to pointers
 *
 ******************************************************************************************/ 
#define 	__IAR_SYSTEMS_ASM__ 
#define 	ENABLE_BIT_DEFINITIONS

#include <iom103.h>
#include <ina90.h>         

//char not_leap(void);

typedef struct{ 
unsigned char second;   			//enter the current time, date, month, and year
unsigned char minute;
unsigned char hour;                                     
unsigned char date;       
unsigned char month;
unsigned int year;      
            }time;
time t;      
                                                        
void C_task main(void)   			// C_task means "main" is never called from another function
{                
           
//    init_rtc();
//    int temp0,temp1;   

     
/*    for(temp0=0;temp0<0x0040;temp0++)   	// Wait for external clock crystal to stabilize
    {
        for(temp1=0;temp1<0xFFFF;temp1++);
    }
*/    DDRB=0xFF;           
    TIMSK &=~((1<<TOIE0)|(1<<OCIE0));     	//Disable TC0 interrupt
    ASSR |= (1<<AS0);           			//set Timer/Counter0 to be asynchronous from the CPU clock 
                                			//with a second external clock(32,768kHz)driving it.  
    TCNT0 = 0x00;
    TCCR0 = 0x05;  //5               		//prescale the timer to be clock source / 128 to make it
                                			//exactly 1 second for every overflow to occur
    while(ASSR&0x07);           			//Wait until TC0 is updated
    TIMSK |= (1<<TOIE0);       				//set 8-bit Timer/Counter0 Overflow Interrupt Enable                             
    _SEI();                     			//set the Global Interrupt Enable Bit  
                              
    while(1)                     
    {
        MCUCR = 0x38;           			//entering sleeping mode: power save mode
        _SLEEP();              				//will wake up from time overflow interrupt  
        _NOP();
        TCCR0=0x05;           				// Write dummy value to Control register
        while(ASSR&0x07);     				//Wait until TC0 is updated
    }            
}

interrupt [TIMER0_OVF_vect] void counter(void) //overflow interrupt vector
{ 
	time *tptr = &t;
    
    if (++tptr->second==60)        			//keep track of time, date, month, and year
    {
        tptr->second=0;
        if (++tptr->minute==60) 
        {
            tptr->minute=0;
            if (++tptr->hour==24)
            {
                tptr->hour=0;
                if (++tptr->date==32)
                {
                    tptr->month++;
                    tptr->date=1;
                }
                else if (tptr->date==31) 
                {                    
                    if ((tptr->month==4 || 6 || 9 || 11)) 
                    {
                        tptr->month++;
                        tptr->date=1;
                    }
                }
                else if (tptr->date==30)
                {
                    if(tptr->month==2)
                    {
                       tptr->month++;
                       tptr->date=1;
                    }
                }              
                else if (tptr->date==29) 
                {
//                    if((tptr->month==2) && (not_leap()))	see comment at not_leap
                    if((tptr->month==2) && (tptr->year%4))
                    {
                        tptr->month++;
                        tptr->date=1;
                    }                
                }                          
                if (tptr->month==13)
                {
                    tptr->month=1;
                    tptr->year++;
                }
            }
        }
    }  
    PORTB=~(((tptr->second&0x01)|tptr->minute<<1)|tptr->hour<<7); 


}  
 /*
Removed this function as it is only needed every 100 year
char not_leap(void)      //check for leap year
{
    if (!(t.year%100))
        return (char)(t.year%400);
    else
        return (char)(t.year%4);
}         
  */
          
    
        
